﻿

Public Class Form_Options

    Private Sub Form_Options_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        Me.Hide()
        e.Cancel = True
    End Sub

    Private Sub Button_Close_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_Close.Click
        Me.Hide()
    End Sub

    Private Sub Form_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Return
        LimitFormPosition(Me)
    End Sub

    Private Sub TrackBar1_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrackBar1.Scroll
        If Math.Abs(TrackBar1.Value - 100) < 5 Then TrackBar1.Value = 100
        Editor_SetZoom(TrackBar1.Value)
    End Sub

    Private Sub ComboBox_Font_SelectedIndexChanged(ByVal sender As System.Object, _
                                                    ByVal e As System.EventArgs) _
                                                            Handles ComboBox_Font.SelectedIndexChanged
        If Not EventsAreEnabled Then Exit Sub
        DirectCast(sender, Control).Refresh()
        Editor_SetCursor_Wait()
        Editor_SetFont(ComboBox_Font.Text)
        Editor_NormalizeAndColorAllLines()
        SaveUserOptions()
        Editor_SetCursor_Normal()
    End Sub

    Private Sub ComboBox_TabSize_SelectedIndexChanged(ByVal sender As System.Object, _
                                                        ByVal e As System.EventArgs) _
                                                            Handles ComboBox_TabSize.SelectedIndexChanged

        If Not EventsAreEnabled Then Exit Sub
        Editor_SetTabSize(ComboBox_TabSize.Text)
        SaveUserOptions()
    End Sub

    Private Sub ColorLabels_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Label_NormalText.Click, _
                                                                                                        Label_QuotedText.Click, _
                                                                                                        Label_Keywords.Click, _
                                                                                                        Label_Comments.Click, _
                                                                                                        Label_Errors.Click, _
                                                                                                        Label_Warnings.Click, _
                                                                                                        Label_TextBack.Click, _
                                                                                                        Label_EditorBack.Click

        If Not EventsAreEnabled Then Exit Sub
        Dim c As Control = DirectCast(sender, Control)
        c.BackColor = EditColor(c.BackColor)
        c.Refresh()
        Editor_SetCursor_Wait()
        SendColors_ToEditor()
        Editor_NormalizeAndColorAllLines()
        SaveUserOptions()
        Editor_SetCursor_Normal()
    End Sub

    Private Sub SendColors_ToEditor()
        Editor_SetColors(Label_NormalText.BackColor, _
                        Label_QuotedText.BackColor, _
                        Label_Keywords.BackColor, _
                        Label_Comments.BackColor, _
                        Label_Errors.BackColor, _
                        Label_Warnings.BackColor, _
                        Label_TextBack.BackColor, _
                        Label_EditorBack.BackColor)
    End Sub

    Private Sub CompilerOptions_Changed(ByVal sender As System.Object, _
                                        ByVal e As System.EventArgs) _
                                                Handles ComboBox_Language.SelectedIndexChanged, _
                                                        ComboBox_WarningLevel.SelectedIndexChanged, _
                                                        CheckBox_HideHeader.CheckedChanged, _
                                                        CheckBox_OptionExplicit.CheckedChanged, _
                                                        CheckBox_OptionStrict.CheckedChanged, _
                                                        CheckBox_OptionOptimize.CheckedChanged

        If Not EventsAreEnabled Then Exit Sub
        DirectCast(sender, Control).Refresh()
        Editor_SetCursor_Wait()
        SendCompilerOptions_ToEditor()
        Editor_NormalizeAndColorAllLines()
        SaveUserOptions()
        Editor_SetCursor_Normal()
    End Sub

    Private Sub CompilerOptions2_Changed(ByVal sender As System.Object, _
                                        ByVal e As System.EventArgs) _
                                              Handles CheckBox_AddDebugInfo.CheckedChanged, _
                                                      CheckBox_UseConsole.CheckedChanged

        If Not EventsAreEnabled Then Exit Sub
        DirectCast(sender, Control).Refresh()
        SendCompilerOptions_ToEditor()
        Editor_NormalizeAndColorAllLines()
        SaveUserOptions()
    End Sub

    Private Sub SendCompilerOptions_ToEditor()
        Compiler_SetOptions(If(ComboBox_Language.Text = "VB", Compiler_Languages.VB, Compiler_Languages.CSharp), _
                            IntVal(ComboBox_WarningLevel.Text), _
                            CheckBox_HideHeader.Checked, _
                            CheckBox_OptionExplicit.Checked, _
                            CheckBox_OptionStrict.Checked, _
                            CheckBox_OptionOptimize.Checked, _
                            CheckBox_AddDebugInfo.Checked, _
                            CheckBox_UseConsole.Checked)
    End Sub

    Friend Sub SendAllOptions_ToEditor()
        ' ---------------------------------------------------- editor
        Editor_SetFont(ComboBox_Font.Text)
        Editor_SetTabSize(ComboBox_TabSize.Text)
        SendColors_ToEditor()
        Editor_NormalizeAndColorAllLines()
        Editor_SetZoom(TrackBar1.Value)
        ' ---------------------------------------------------- compiler
        SendCompilerOptions_ToEditor()
    End Sub


    Private Function EditColor(ByVal col As Color) As Color
        Dim cd As ColorDialog
        cd = New ColorDialog()
        cd.AllowFullOpen = True
        cd.FullOpen = True
        cd.ShowHelp = True
        cd.AnyColor = True
        cd.SolidColorOnly = False
        cd.Color = col
        cd.CustomColors = CustomColors
        If (cd.ShowDialog() = Windows.Forms.DialogResult.OK) Then
            col = cd.Color
            CustomColors = cd.CustomColors
            SaveUserOptions()
        End If
        Return col
    End Function

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        If MsgBox("Set all options to default ?", MsgBoxStyle.Question Or MsgBoxStyle.YesNo, "Options") = MsgBoxResult.Yes Then
            SetDefaultOptions()
            SendAllOptions_ToEditor()
            SaveUserOptions()
        End If
    End Sub

    Friend Sub AutoDetectLanguage(ByRef s As String, ByVal FileName As String)
        ' -------------------------------------------------------------------- VB
        Dim nImports As Int32 = STR_Count(s, "Imports", CompareMethod.Text)
        Dim nSub As Int32 = STR_Count(s, "Sub", CompareMethod.Text)
        Dim nFunc As Int32 = STR_Count(s, "Function", CompareMethod.Text)
        ' -------------------------------------------------------------------- CS
        Dim nUsing As Int32 = STR_Count(s, "Using", CompareMethod.Text)
        Dim nVoid As Int32 = STR_Count(s, "Void", CompareMethod.Text)
        Dim nPointComma As Int32 = STR_CountChar(s, ";") \ 10
        '
        'Debug.Print(nImports.ToString & vbTab & nSub.ToString & vbTab & nFunc.ToString)
        'Debug.Print(nUsing.ToString & vbTab & nVoid.ToString & vbTab & nPointComma.ToString)
        '
        EventsAreEnabled = False
        If nImports + nUsing > 0 Then
            CheckBox_HideHeader.Checked = False
        Else
            CheckBox_HideHeader.Checked = True
        End If

        'If FileName.EndsWith(".vb", StringComparison.InvariantCultureIgnoreCase) Then
        '    Combo_SetIndex_FromString(ComboBox_Language, "VB")
        'ElseIf FileName.EndsWith(".cs", StringComparison.InvariantCultureIgnoreCase) Then
        '    Combo_SetIndex_FromString(ComboBox_Language, "C#")
        'Else
        If nImports + nSub + nFunc >= nUsing + nVoid + nPointComma Then
            Combo_SetIndex_FromString(ComboBox_Language, "VB")
        Else
            Combo_SetIndex_FromString(ComboBox_Language, "C#")
        End If
        'End If

        SendCompilerOptions_ToEditor()
        EventsAreEnabled = True
    End Sub

    Private Function STR_Count(ByRef StringToCheck As String, _
                              ByVal StringToFind As String, _
                              ByVal Compare_Method As CompareMethod) As Int32

        Dim Occurrences As Int32
        Dim Start As Int32
        Dim Found As Int32
        Do
            Start = Found + 1
            Found = InStr(Start, StringToCheck, StringToFind, Compare_Method)
            '
            If Found = 0 Then Exit Do
            '
            ' ----------------------------------------------------------- exclude commented lines 
            Dim linestart As Int32 = InStrRev(StringToCheck, vbLf, Found)
            If InStr(StringToCheck.Substring(linestart, Found - linestart), "//") = 0 AndAlso _
                InStr(StringToCheck.Substring(linestart, Found - linestart), "'") = 0 Then
                Occurrences += 1
            End If
        Loop
        Return Occurrences
    End Function

    Private Function STR_CountChar(ByVal StringToCheck As String, _
                                    ByVal CharToFind As String) As Int32

        If StringToCheck Is Nothing Then
            Return 0
        Else
            Return StringToCheck.Split(CChar(CharToFind)).Length - 1
        End If
    End Function

    Friend Sub SetDefaultOptions()
        ' ------------------------------------------------------------ editor options
        Combo_SetIndex_FromString(ComboBox_Font, "Courier New")
        Combo_SetIndex_FromString(ComboBox_TabSize, "4")
        Label_NormalText.BackColor = Color.Black
        Label_QuotedText.BackColor = Color.FromArgb(180, 22, 22) 'Color.DarkRed
        Label_Keywords.BackColor = Color.FromArgb(0, 0, 255) 'Color.Blue
        Label_Comments.BackColor = Color.FromArgb(0, 128, 0) 'Color.DarkGreen
        Label_Errors.BackColor = Color.FromArgb(255, 220, 210)
        Label_Warnings.BackColor = Color.FromArgb(220, 255, 210)
        Label_TextBack.BackColor = Color.FromArgb(245, 245, 245)
        Label_EditorBack.BackColor = Color.FromArgb(255, 255, 255)
        TrackBar1.Value = 100
        ' ------------------------------------------------------------ compiler options
        Combo_SetIndex_FromString(ComboBox_WarningLevel, "4")
        Combo_SetIndex_FromString(ComboBox_Language, "VB")
        CheckBox_OptionExplicit.Checked = True
        CheckBox_OptionStrict.Checked = True
        CheckBox_OptionOptimize.Checked = False
        CheckBox_AddDebugInfo.Checked = False
        CheckBox_UseConsole.Checked = False
    End Sub

End Class