﻿Module Module_SaveLoad

    Friend CustomColors(-1) As Int32
    Friend EventsAreEnabled As Boolean = False


    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' =====================================================================================
    '  Helper functions  
    ' =====================================================================================
    'Public Function StrToFloat(ByVal s As String) As Double
    '    StrToFloat = Val(Replace(s, ",", "."))
    'End Function
    Public Function FloatToStr(ByVal n As Double) As String
        FloatToStr = Trim(Str(n))
        If Strings.Left(FloatToStr, 1) = "." Then FloatToStr = "0" & FloatToStr
    End Function
    Public Function FloatToStr_3(ByVal n As Double) As String
        FloatToStr_3 = Format(n, "0.000")
        FloatToStr_3 = Replace(FloatToStr_3, ",", ".")
    End Function
    Public Function BoolToStr(ByVal b As Boolean) As String
        If b Then
            BoolToStr = "1"
        Else
            BoolToStr = "0"
        End If
    End Function
    Public Function ExtractBoolean(ByRef s As String) As Boolean
        ExtractBoolean = Val(s) <> 0
    End Function
    Public Function ExtractInteger(ByRef s As String, ByVal min As Integer, ByVal max As Integer) As Int32
        ExtractInteger = IntVal(s)
        If ExtractInteger < min Then ExtractInteger = min
        If ExtractInteger > max Then ExtractInteger = max
    End Function
    Public Function ExtractDouble(ByRef s As String, ByVal min As Double, ByVal max As Double) As Double
        ExtractDouble = Val(s)
        If ExtractDouble < min Then ExtractDouble = min
        If ExtractDouble > max Then ExtractDouble = max
    End Function
    Public Function ExtractSingle(ByRef s As String, ByVal min As Single, ByVal max As Single) As Single
        ExtractSingle = CSng(Val(s))
        If ExtractSingle < min Then ExtractSingle = min
        If ExtractSingle > max Then ExtractSingle = max
    End Function
    Public Function IntVal(ByVal s As String) As Int32
        Return CInt(Val(s))
    End Function
    Public Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    ' =====================================================================================
    '  Colors 
    ' =====================================================================================
    Private Function Color_ToHtmlString(ByVal col As Color) As String
        Return Drawing.ColorTranslator.ToHtml(col)
    End Function
    Private Function Color_FromHtmlString(ByVal HtmlColorString As String) As Color
        On Error Resume Next
        Return Drawing.ColorTranslator.FromHtml(HtmlColorString)
    End Function
    Private Function Color_IntToHtmlString(ByVal Win32Col As Int32) As String
        Dim col As Color = Drawing.ColorTranslator.FromWin32(Win32Col)
        Return Drawing.ColorTranslator.ToHtml(col)
    End Function
    Private Function Color_IntFromHtmlString(ByVal HtmlColorString As String) As Int32
        On Error Resume Next
        Dim col As Color = Drawing.ColorTranslator.FromHtml(HtmlColorString)
        Return Drawing.ColorTranslator.ToWin32(col)
    End Function

    ' =====================================================================================
    '  Files  
    ' =====================================================================================
    Public Sub File_KillFile(ByRef FileName As String)
        On Error Resume Next
        If Not My.Computer.FileSystem.FileExists(FileName) Then Exit Sub
        SetAttr(FileName, FileAttribute.Normal)
        Kill(FileName)
    End Sub


    ' =====================================================================================
    '  Combo functions  
    ' =====================================================================================
    Friend Sub Combo_SetIndex_FromString(ByVal combo As ComboBox, ByVal str As String)
        Dim old As Boolean = EventsAreEnabled
        EventsAreEnabled = False
        If str = Nothing Then str = ""
        With combo
            For i As Int32 = 0 To .Items.Count - 1
                If .Items(i).ToString = str Then
                    .SelectedIndex = i
                    Exit For
                End If
            Next
        End With
        EventsAreEnabled = old
    End Sub



    ' ==================================================================================================
    '  SAVE LOAD -- USER OPTIONS
    ' ==================================================================================================
    Friend Sub SaveUserOptions()
        '
        Dim TempFile As String = My.Application.Info.DirectoryPath & "\UserOptions_Temp.txt"
        File_KillFile(TempFile)
        Dim f As Int32 = FreeFile()
        '
        Try
            FileOpen(f, TempFile, OpenMode.Output, OpenAccess.Write)
            PrintLine(f, " Form main")
            PrintLine(f, "======================================================================")
            ' ------------------------------------------------------------------------------ FORM MAIN BOUNDS
            ' (The value of the RestoreBounds property is valid only 
            '   when the WindowState property of the Form class is not equal to Normal)
            Dim r As Rectangle
            If Form_Main.WindowState = FormWindowState.Normal Then
                r = Form_Main.Bounds
            Else
                r = Form_Main.RestoreBounds
            End If
            'Dim a As Int32 = 0 : a = a \ a
            PrintLine(f, "Top=", TAB(22), FloatToStr(r.Top))
            PrintLine(f, "Left=", TAB(22), FloatToStr(r.Left))
            PrintLine(f, "Width=", TAB(22), FloatToStr(r.Width))
            PrintLine(f, "Height=", TAB(22), FloatToStr(r.Height))
            PrintLine(f, "Split1=", TAB(22), FloatToStr(Form_Main.SplitContainer1.SplitterDistance))
            PrintLine(f, "Split2=", TAB(22), FloatToStr(Form_Main.SplitContainer2.SplitterDistance))
            '
            PrintLine(f, "")
            PrintLine(f, " Editor options")
            PrintLine(f, "======================================================================")
            PrintLine(f, "Font=", TAB(22), Form_Options.ComboBox_Font.Text)
            PrintLine(f, "TabSize=", TAB(22), Form_Options.ComboBox_TabSize.Text)
            PrintLine(f, "Color_NormalText=", TAB(22), Color_ToHtmlString(Form_Options.Label_NormalText.BackColor))
            PrintLine(f, "Color_QuotedText=", TAB(22), Color_ToHtmlString(Form_Options.Label_QuotedText.BackColor))
            PrintLine(f, "Color_Keywords=", TAB(22), Color_ToHtmlString(Form_Options.Label_Keywords.BackColor))
            PrintLine(f, "Color_Comments=", TAB(22), Color_ToHtmlString(Form_Options.Label_Comments.BackColor))
            PrintLine(f, "Color_Errors=", TAB(22), Color_ToHtmlString(Form_Options.Label_Errors.BackColor))
            PrintLine(f, "Color_Warnings=", TAB(22), Color_ToHtmlString(Form_Options.Label_Warnings.BackColor))
            PrintLine(f, "Color_TextBack=", TAB(22), Color_ToHtmlString(Form_Options.Label_TextBack.BackColor))
            PrintLine(f, "Color_EditorBack=", TAB(22), Color_ToHtmlString(Form_Options.Label_EditorBack.BackColor))
            PrintLine(f, "Zoom=", TAB(22), Form_Options.TrackBar1.Value.ToString)
            PrintLine(f, "FileName=", TAB(22), Editor_FileName)
            '
            PrintLine(f, "")
            PrintLine(f, " Compiler options")
            PrintLine(f, "======================================================================")
            PrintLine(f, "Language=", TAB(22), Form_Options.ComboBox_Language.Text)
            PrintLine(f, "WarningLevel=", TAB(22), Form_Options.ComboBox_WarningLevel.Text)
            PrintLine(f, "OptionHideHeader=", TAB(22), BoolToStr(Form_Options.CheckBox_HideHeader.Checked))
            PrintLine(f, "OptionExplicit=", TAB(22), BoolToStr(Form_Options.CheckBox_OptionExplicit.Checked))
            PrintLine(f, "OptionStrict=", TAB(22), BoolToStr(Form_Options.CheckBox_OptionStrict.Checked))
            PrintLine(f, "OptionOptimize=", TAB(22), BoolToStr(Form_Options.CheckBox_OptionOptimize.Checked))
            PrintLine(f, "AddDebugInfo=", TAB(22), BoolToStr(Form_Options.CheckBox_AddDebugInfo.Checked))
            PrintLine(f, "UseConsole=", TAB(22), BoolToStr(Form_Options.CheckBox_UseConsole.Checked))
            '
            PrintLine(f, "")
            PrintLine(f, " Color dialog - custom colors")
            PrintLine(f, "======================================================================")
            '
            For i As Int32 = 0 To CustomColors.Count - 1
                'PrintLine(f, "CustomColor=", TAB(22), CustomColors(i).ToString)
                PrintLine(f, "CustomColor=", TAB(22), Color_IntToHtmlString(CustomColors(i)))
            Next
            PrintLine(f, "")
            '
            FileClose(f)

            ' ------------------------------------------------------------------ copy the temp file on the main file
            My.Computer.FileSystem.CopyFile(TempFile, My.Application.Info.DirectoryPath & "\UserOptions.txt", True)
            File_KillFile(TempFile)
        Catch
        End Try
        Try
            FileClose(f)
        Catch
        End Try
        File_KillFile(TempFile)
    End Sub


    Friend Sub LoadUserOptions()
        EventsAreEnabled = False
        ' ------------------------------------------------------------------------------- set defauts
        Form_Options.SetDefaultOptions()
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        ReDim CustomColors(-1)
        Dim l As String
        Dim file As String
        file = My.Application.Info.DirectoryPath & "\UserOptions.txt"
        If My.Computer.FileSystem.FileExists(file) Then
            Dim f As Int32 = FreeFile()
            FileOpen(f, file, OpenMode.Input, OpenAccess.Read, OpenShare.Shared)
            Do While Not EOF(f)
                l = LineInput(f)
                Select Case ExtractParamName(l)
                    ' ------------------------------------------------- main form
                    Case "Top" : Form_Main.Top = ExtractInteger(l, 0, 10000)
                    Case "Left" : Form_Main.Left = ExtractInteger(l, 0, 10000)
                    Case "Width" : Form_Main.Width = ExtractInteger(l, 0, 10000)
                    Case "Height" : Form_Main.Height = ExtractInteger(l, 0, 10000)
                    Case "Split1" : Form_Main.SplitContainer1.SplitterDistance = ExtractInteger(l, 0, 10000)
                    Case "Split2" : Form_Main.SplitContainer2.SplitterDistance = ExtractInteger(l, 0, 10000)
                        '
                        ' --------------------------------------------- editor options
                    Case "Font" : Combo_SetIndex_FromString(Form_Options.ComboBox_Font, l)
                    Case "TabSize" : Combo_SetIndex_FromString(Form_Options.ComboBox_TabSize, l)
                    Case "Color_NormalText" : Form_Options.Label_NormalText.BackColor = Color_FromHtmlString(l)
                    Case "Color_QuotedText" : Form_Options.Label_QuotedText.BackColor = Color_FromHtmlString(l)
                    Case "Color_Keywords" : Form_Options.Label_Keywords.BackColor = Color_FromHtmlString(l)
                    Case "Color_Comments" : Form_Options.Label_Comments.BackColor = Color_FromHtmlString(l)
                    Case "Color_Errors" : Form_Options.Label_Errors.BackColor = Color_FromHtmlString(l)
                    Case "Color_Warnings" : Form_Options.Label_Warnings.BackColor = Color_FromHtmlString(l)
                    Case "Color_TextBack" : Form_Options.Label_TextBack.BackColor = Color_FromHtmlString(l)
                    Case "Color_EditorBack" : Form_Options.Label_EditorBack.BackColor = Color_FromHtmlString(l)
                    Case "Zoom" : Form_Options.TrackBar1.Value = IntVal(l)
                    Case "FileName" : Editor_FileName = l
                        '
                        ' --------------------------------------------- compiler options
                    Case "Language" : Combo_SetIndex_FromString(Form_Options.ComboBox_Language, l)
                    Case "WarningLevel" : Combo_SetIndex_FromString(Form_Options.ComboBox_WarningLevel, l)
                    Case "OptionHideHeader" : Form_Options.CheckBox_HideHeader.Checked = CBool(l)
                    Case "OptionExplicit" : Form_Options.CheckBox_OptionExplicit.Checked = CBool(l)
                    Case "OptionStrict" : Form_Options.CheckBox_OptionStrict.Checked = CBool(l)
                    Case "OptionOptimize" : Form_Options.CheckBox_OptionOptimize.Checked = CBool(l)
                    Case "AddDebugInfo" : Form_Options.CheckBox_AddDebugInfo.Checked = CBool(l)
                    Case "UseConsole" : Form_Options.CheckBox_UseConsole.Checked = CBool(l)
                        '
                        ' --------------------------------------------- custom colors
                    Case "CustomColor"
                        Dim index As Int32 = CustomColors.Count
                        ReDim Preserve CustomColors(index)
                        'CustomColors(index) = IntVal(l)
                        CustomColors(index) = Color_IntFromHtmlString(l)
                End Select
            Loop
            FileClose(f)
        End If
        LimitFormPosition(Form_Main)
        Form_Options.SendAllOptions_ToEditor()
        EventsAreEnabled = True
    End Sub


End Module
